@@ -49,6 +49,7 @@
 #include "qquickpopupitem_p_p.h"
 #include "qquickapplicationwindow_p.h"
 #include "qquickdeferredexecute_p_p.h"
+#include "qquickcontentitem_p.h"
 
 #if QT_CONFIG(accessibility)
 #include <QtQuick/private/qquickaccessibleattached_p.h>
@@ -139,7 +140,8 @@ Q_LOGGING_CATEGORY(lcItemManagement, "qt.quick.controls.control.itemmanagement")
     }
     \endcode
 
-    \sa ApplicationWindow, Container
+    \sa ApplicationWindow, Container, {Using Qt Quick Controls types in
+        property declarations}
 */
 
 const QQuickItemPrivate::ChangeTypes QQuickControlPrivate::ImplicitSizeChanges = QQuickItemPrivate::ImplicitWidth | QQuickItemPrivate::ImplicitHeight | QQuickItemPrivate::Destroyed;
@@ -158,9 +160,6 @@ QQuickControlPrivate::QQuickControlPrivate()
 
 QQuickControlPrivate::~QQuickControlPrivate()
 {
-#if QT_CONFIG(accessibility)
-    QAccessible::removeActivationObserver(this);
-#endif
 }
 
 void QQuickControlPrivate::init()
@@ -175,15 +174,15 @@ bool QQuickControlPrivate::acceptTouch(const QTouchEvent::TouchPoint &point)
     if (point.id() == touchId)
         return true;
 
-    if (touchId == -1 && point.state() == Qt::TouchPointPressed) {
+    if (touchId == -1 && point.state() == QEventPoint::Pressed) {
         touchId = point.id();
         return true;
     }
 
     // If the control is on a Flickable that has a pressDelay, then the press is never
     // sent as a touch event, therefore we need to check for this case.
-    if (touchId == -1 && pressWasTouch && point.state() == Qt::TouchPointReleased &&
-        point.pos() == previousPressPos) {
+    if (touchId == -1 && pressWasTouch && point.state() == QEventPoint::Released &&
+        point.position() == previousPressPos) {
         return true;
     }
     return false;
@@ -489,6 +488,11 @@ void QQuickControlPrivate::updateImplicitContentSize()
         emit q->implicitContentHeightChanged();
 }
 
+QPalette QQuickControlPrivate::defaultPalette() const
+{
+    return QQuickTheme::palette(QQuickTheme::System);
+}
+
 #if QT_CONFIG(accessibility)
 void QQuickControlPrivate::accessibilityActiveChanged(bool active)
 {
@@ -556,16 +560,10 @@ void QQuickControlPrivate::inheritFont(const QFont &font)
 {
     Q_Q(QQuickControl);
     QFont parentFont = extra.isAllocated() ? extra->requestedFont.resolve(font) : font;
-    parentFont.resolve(extra.isAllocated() ? extra->requestedFont.resolve() | font.resolve() : font.resolve());
+    parentFont.setResolveMask(extra.isAllocated() ? extra->requestedFont.resolveMask() | font.resolveMask() : font.resolveMask());
 
     const QFont defaultFont = q->defaultFont();
     QFont resolvedFont = parentFont.resolve(defaultFont);
-    // Since resolving the font will put the family() into the
-    // families() list if it is empty then we need to unset it
-    // so it does not act as if the font has changed (when it
-    // has not actually changed)
-    if (defaultFont.families().isEmpty())
-        resolvedFont.setFamilies(QStringList());
 
     setFont_helper(resolvedFont);
 }
@@ -607,97 +605,6 @@ void QQuickControlPrivate::updateFontRecur(QQuickItem *item, const QFont &font)
     }
 }
 
-/*!
-    \internal
-
-    Returns the palette that the item inherits from its ancestors and
-    QGuiApplication::palette.
-*/
-QPalette QQuickControlPrivate::parentPalette(const QQuickItem *item)
-{
-    QQuickItem *p = item->parentItem();
-    while (p) {
-        if (QQuickControl *control = qobject_cast<QQuickControl *>(p))
-            return control->palette();
-        else if (QQuickLabel *label = qobject_cast<QQuickLabel *>(p))
-            return label->palette();
-        else if (QQuickTextField *textField = qobject_cast<QQuickTextField *>(p))
-            return textField->palette();
-        else if (QQuickTextArea *textArea = qobject_cast<QQuickTextArea *>(p))
-            return textArea->palette();
-
-        p = p->parentItem();
-    }
-
-    if (QQuickApplicationWindow *window = qobject_cast<QQuickApplicationWindow *>(item->window()))
-        return window->palette();
-
-    return QQuickTheme::palette(QQuickTheme::System);
-}
-
-/*!
-    \internal
-
-    Determine which palette is implicitly imposed on this control by its ancestors
-    and QGuiApplication::palette, resolve this against its own palette (attributes from
-    the implicit palette are copied over). Then propagate this palette to this
-    control's children.
-*/
-void QQuickControlPrivate::resolvePalette()
-{
-    Q_Q(QQuickControl);
-    inheritPalette(parentPalette(q));
-}
-
-void QQuickControlPrivate::inheritPalette(const QPalette &palette)
-{
-    Q_Q(QQuickControl);
-    QPalette parentPalette = extra.isAllocated() ? extra->requestedPalette.resolve(palette) : palette;
-    parentPalette.resolve(extra.isAllocated() ? extra->requestedPalette.resolve() | palette.resolve() : palette.resolve());
-
-    const QPalette defaultPalette = q->defaultPalette();
-    const QPalette resolvedPalette = parentPalette.resolve(defaultPalette);
-
-    setPalette_helper(resolvedPalette);
-}
-
-/*!
-    \internal
-
-    Assign \a palette to this control, and propagate it to all children.
-*/
-void QQuickControlPrivate::updatePalette(const QPalette &palette)
-{
-    Q_Q(QQuickControl);
-    QPalette oldPalette = resolvedPalette;
-    resolvedPalette = palette;
-
-    if (oldPalette != palette)
-        q->paletteChange(palette, oldPalette);
-
-    QQuickControlPrivate::updatePaletteRecur(q, palette);
-
-    if (oldPalette != palette)
-        emit q->paletteChanged();
-}
-
-void QQuickControlPrivate::updatePaletteRecur(QQuickItem *item, const QPalette &palette)
-{
-    const auto childItems = item->childItems();
-    for (QQuickItem *child : childItems) {
-        if (QQuickControl *control = qobject_cast<QQuickControl *>(child))
-            QQuickControlPrivate::get(control)->inheritPalette(palette);
-        else if (QQuickLabel *label = qobject_cast<QQuickLabel *>(child))
-            QQuickLabelPrivate::get(label)->inheritPalette(palette);
-        else if (QQuickTextArea *textArea = qobject_cast<QQuickTextArea *>(child))
-            QQuickTextAreaPrivate::get(textArea)->inheritPalette(palette);
-        else if (QQuickTextField *textField = qobject_cast<QQuickTextField *>(child))
-            QQuickTextFieldPrivate::get(textField)->inheritPalette(palette);
-        else
-            QQuickControlPrivate::updatePaletteRecur(child, palette);
-    }
-}
-
 QLocale QQuickControlPrivate::calcLocale(const QQuickItem *item)
 {
     const QQuickItem *p = item;
@@ -968,6 +875,9 @@ QQuickControl::~QQuickControl()
     Q_D(QQuickControl);
     d->removeImplicitSizeListener(d->background, QQuickControlPrivate::ImplicitSizeChanges | QQuickItemPrivate::Geometry);
     d->removeImplicitSizeListener(d->contentItem);
+#if QT_CONFIG(accessibility)
+    QAccessible::removeActivationObserver(d);
+#endif
 }
 
 void QQuickControl::itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &value)
@@ -976,7 +886,6 @@ void QQuickControl::itemChange(QQuickItem::ItemChange change, const QQuickItem::
     QQuickItem::itemChange(change, value);
     switch (change) {
     case ItemEnabledHasChanged:
-        emit paletteChanged();
         enabledChange();
         break;
     case ItemVisibleHasChanged:
@@ -989,7 +898,6 @@ void QQuickControl::itemChange(QQuickItem::ItemChange change, const QQuickItem::
     case ItemParentHasChanged:
         if ((change == ItemParentHasChanged && value.item) || (change == ItemSceneChange && value.window)) {
             d->resolveFont();
-            d->resolvePalette();
             if (!d->hasLocale)
                 d->updateLocale(QQuickControlPrivate::calcLocale(d->parentItem), false); // explicit=false
 #if QT_CONFIG(quicktemplates2_hover)
@@ -1060,7 +968,7 @@ QFont QQuickControl::font() const
 void QQuickControl::setFont(const QFont &font)
 {
     Q_D(QQuickControl);
-    if (d->extra.value().requestedFont.resolve() == font.resolve() && d->extra.value().requestedFont == font)
+    if (d->extra.value().requestedFont.resolveMask() == font.resolveMask() && d->extra.value().requestedFont == font)
         return;
 
     d->extra.value().requestedFont = font;
@@ -1529,7 +1437,7 @@ void QQuickControl::setHoverEnabled(bool enabled)
 
     d->updateHoverEnabled(enabled, true); // explicit=true
 #else
-    Q_UNUSED(enabled)
+    Q_UNUSED(enabled);
 #endif
 }
 
@@ -1712,75 +1620,6 @@ void QQuickControl::resetBaselineOffset()
     d->updateBaselineOffset();
 }
 
-/*!
-    \since QtQuick.Controls 2.3 (Qt 5.10)
-    \qmlproperty palette QtQuick.Controls::Control::palette
-
-    This property holds the palette currently set for the control.
-
-    This property describes the control's requested palette. The palette is used by the control's
-    style when rendering standard components, and is available as a means to ensure that custom
-    controls can maintain consistency with the native platform's native look and feel. It's common
-    that different platforms, or different styles, define different palettes for an application.
-
-    The default palette depends on the system environment. ApplicationWindow maintains a system/theme
-    palette which serves as a default for all controls. There may also be special palette defaults for
-    certain types of controls. You can also set the default palette for controls by either:
-
-    \list
-    \li passing a custom palette to QGuiApplication::setPalette(), before loading any QML; or
-    \li specifying the colors in the \l {Qt Quick Controls 2 Configuration File}{qtquickcontrols2.conf file}.
-    \endlist
-
-    Control propagates explicit palette properties from parent to children. If you change a specific
-    property on a control's palette, that property propagates to all of the control's children,
-    overriding any system defaults for that property.
-
-    \code
-    Page {
-        palette.text: "red"
-
-        Column {
-            Label {
-                text: qsTr("This will use red color...")
-            }
-
-            Switch {
-                text: qsTr("... and so will this")
-            }
-        }
-    }
-    \endcode
-
-    For the full list of available palette colors, see the
-    \l {qtquickcontrols2-palette}{palette QML Basic Type} documentation.
-
-    \sa ApplicationWindow::palette, Popup::palette
-*/
-QPalette QQuickControl::palette() const
-{
-    Q_D(const QQuickControl);
-    QPalette palette = d->resolvedPalette;
-    if (!isEnabled())
-        palette.setCurrentColorGroup(QPalette::Disabled);
-    return palette;
-}
-
-void QQuickControl::setPalette(const QPalette &palette)
-{
-    Q_D(QQuickControl);
-    if (d->extra.value().requestedPalette.resolve() == palette.resolve() && d->extra.value().requestedPalette == palette)
-        return;
-
-    d->extra.value().requestedPalette = palette;
-    d->resolvePalette();
-}
-
-void QQuickControl::resetPalette()
-{
-    setPalette(QPalette());
-}
-
 /*!
     \since QtQuick.Controls 2.5 (Qt 5.12)
     \qmlproperty real QtQuick.Controls::Control::horizontalPadding
@@ -2058,7 +1897,6 @@ void QQuickControl::classBegin()
     Q_D(QQuickControl);
     QQuickItem::classBegin();
     d->resolveFont();
-    d->resolvePalette();
 }
 
 void QQuickControl::componentComplete()
@@ -2087,11 +1925,6 @@ QFont QQuickControl::defaultFont() const
     return QQuickTheme::font(QQuickTheme::System);
 }
 
-QPalette QQuickControl::defaultPalette() const
-{
-    return QQuickTheme::palette(QQuickTheme::System);
-}
-
 void QQuickControl::focusInEvent(QFocusEvent *event)
 {
     QQuickItem::focusInEvent(event);
@@ -2115,7 +1948,7 @@ void QQuickControl::hoverEnterEvent(QHoverEvent *event)
 void QQuickControl::hoverMoveEvent(QHoverEvent *event)
 {
     Q_D(QQuickControl);
-    setHovered(d->hoverEnabled && contains(event->pos()));
+    setHovered(d->hoverEnabled && contains(event->position()));
     event->setAccepted(d->hoverEnabled);
 }
 
@@ -2130,10 +1963,10 @@ void QQuickControl::hoverLeaveEvent(QHoverEvent *event)
 void QQuickControl::mousePressEvent(QMouseEvent *event)
 {
     Q_D(QQuickControl);
-    d->handlePress(event->localPos());
+    d->handlePress(event->position());
     if (event->source() == Qt::MouseEventSynthesizedByQt) {
         d->pressWasTouch = true;
-        d->previousPressPos = event->localPos();
+        d->previousPressPos = event->position();
     }
     event->accept();
 }
@@ -2141,14 +1974,14 @@ void QQuickControl::mousePressEvent(QMouseEvent *event)
 void QQuickControl::mouseMoveEvent(QMouseEvent *event)
 {
     Q_D(QQuickControl);
-    d->handleMove(event->localPos());
+    d->handleMove(event->position());
     event->accept();
 }
 
 void QQuickControl::mouseReleaseEvent(QMouseEvent *event)
 {
     Q_D(QQuickControl);
-    d->handleRelease(event->localPos());
+    d->handleRelease(event->position());
     event->accept();
 }
 
@@ -2166,19 +1999,19 @@ void QQuickControl::touchEvent(QTouchEvent *event)
     case QEvent::TouchBegin:
     case QEvent::TouchUpdate:
     case QEvent::TouchEnd:
-        for (const QTouchEvent::TouchPoint &point : event->touchPoints()) {
+        for (const QTouchEvent::TouchPoint &point : event->points()) {
             if (!d->acceptTouch(point))
                 continue;
 
             switch (point.state()) {
-            case Qt::TouchPointPressed:
-                d->handlePress(point.pos());
+            case QEventPoint::Pressed:
+                d->handlePress(point.position());
                 break;
-            case Qt::TouchPointMoved:
-                d->handleMove(point.pos());
+            case QEventPoint::Updated:
+                d->handleMove(point.position());
                 break;
-            case Qt::TouchPointReleased:
-                d->handleRelease(point.pos());
+            case QEventPoint::Released:
+                d->handleRelease(point.position());
                 break;
             default:
                 break;
@@ -2214,10 +2047,10 @@ void QQuickControl::wheelEvent(QWheelEvent *event)
 }
 #endif
 
-void QQuickControl::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
+void QQuickControl::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     Q_D(QQuickControl);
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
+    QQuickItem::geometryChange(newGeometry, oldGeometry);
     d->resizeBackground();
     d->resizeContent();
     if (!qFuzzyCompare(newGeometry.width(), oldGeometry.width()))
@@ -2274,12 +2107,6 @@ void QQuickControl::localeChange(const QLocale &newLocale, const QLocale &oldLoc
     Q_UNUSED(oldLocale);
 }
 
-void QQuickControl::paletteChange(const QPalette &newPalette, const QPalette &oldPalette)
-{
-    Q_UNUSED(newPalette);
-    Q_UNUSED(oldPalette);
-}
-
 void QQuickControl::insetChange(const QMarginsF &newInset, const QMarginsF &oldInset)
 {
     Q_D(QQuickControl);
@@ -2322,7 +2149,7 @@ void QQuickControl::maybeSetAccessibleName(const QString &name)
             accessibleAttached->setNameImplicitly(name);
     }
 #else
-    Q_UNUSED(name)
+    Q_UNUSED(name);
 #endif
 }
 
@@ -2332,7 +2159,7 @@ QVariant QQuickControl::accessibleProperty(const char *propertyName)
     if (QAccessible::isActive())
         return QQuickAccessibleAttached::property(this, propertyName);
 #endif
-    Q_UNUSED(propertyName)
+    Q_UNUSED(propertyName);
     return QVariant();
 }
 
@@ -2342,8 +2169,8 @@ bool QQuickControl::setAccessibleProperty(const char *propertyName, const QVaria
     if (QAccessible::isActive())
         return QQuickAccessibleAttached::setProperty(this, propertyName, value);
 #endif
-    Q_UNUSED(propertyName)
-    Q_UNUSED(value)
+    Q_UNUSED(propertyName);
+    Q_UNUSED(value);
     return false;
 }
 
