@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2017 The Qt Company Ltd.
+** Copyright (C) 2020 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -45,6 +45,9 @@
 
 #include <QtGui/qevent.h>
 #include <QtGui/qcursor.h>
+#if QT_CONFIG(shortcut)
+#include <QtGui/qkeysequence.h>
+#endif
 #include <QtGui/qpa/qplatformintegration.h>
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtQml/qqmlcontext.h>
@@ -53,7 +56,7 @@
 #include <QtQml/private/qv4scopedvalue_p.h>
 #include <QtQml/private/qv4variantobject_p.h>
 #include <QtQml/private/qv4qobjectwrapper_p.h>
-#include <QtQml/private/qqmlobjectmodel_p.h>
+#include <private/qqmlobjectmodel_p.h>
 #include <QtQuick/private/qquickitem_p.h>
 #include <QtQuick/private/qquickitemchangelistener_p.h>
 #include <QtQuick/private/qquickitemview_p.h>
@@ -68,7 +71,7 @@ static const int SUBMENU_DELAY = 225;
 /*!
     \qmltype Menu
     \inherits Popup
-    \instantiates QQuickMenu
+//!     \instantiates QQuickMenu
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-menus
@@ -170,6 +173,14 @@ static const int SUBMENU_DELAY = 225;
     Although \l {MenuItem}{MenuItems} are most commonly used with Menu, it can
     contain any type of item.
 
+    \section1 Margins
+
+    As it is inherited from Popup, Menu supports \l {Popup::}{margins}. By
+    default, all of the built-in styles specify \c 0 for Menu's margins to
+    ensure that the menu is kept within the bounds of the window. To allow a
+    menu to go outside of the window (to animate it moving into view, for
+    example), set the margins property to \c -1.
+
     \sa {Customizing Menu}, MenuItem, {Menu Controls}, {Popup Controls}
 */
 
@@ -178,12 +189,13 @@ static const int SUBMENU_DELAY = 225;
 
     This property holds whether the popup wants focus.
 
-    When the popup actually receives focus, \l activeFocus will be \c true.
-    For more information, see \l {Keyboard Focus in Qt Quick}.
+    When the popup actually receives focus, \l{Popup::}{activeFocus}
+    will be \c true. For more information, see
+    \l {Keyboard Focus in Qt Quick}.
 
     The default value is \c false.
 
-    \sa activeFocus
+    \sa {Popup::}{activeFocus}
 */
 
 static const QQuickPopup::ClosePolicy cascadingSubMenuClosePolicy = QQuickPopup::CloseOnEscape | QQuickPopup::CloseOnPressOutsideParent;
@@ -207,8 +219,13 @@ public:
 
 QQuickMenuPrivate::QQuickMenuPrivate()
 {
-    Q_Q(QQuickMenu);
     cascade = shouldCascade();
+}
+
+void QQuickMenuPrivate::init()
+{
+    Q_Q(QQuickMenu);
+    QQuickPopupPrivate::init();
     contentModel = new QQmlObjectModel(q);
 }
 
@@ -282,8 +299,8 @@ QQuickItem *QQuickMenuPrivate::beginCreateItem()
     QQuickItem *item = qobject_cast<QQuickItem *>(object);
     if (!item)
         delete object;
-
-    QQml_setParent_noEvent(item, q);
+    else
+        QQml_setParent_noEvent(item, q);
 
     return item;
 }
@@ -463,7 +480,7 @@ void QQuickMenuPrivate::onItemHovered()
 {
     Q_Q(QQuickMenu);
     QQuickAbstractButton *button = qobject_cast<QQuickAbstractButton *>(q->sender());
-    if (!button || !button->isHovered() || QQuickAbstractButtonPrivate::get(button)->touchId != -1)
+    if (!button || !button->isHovered() || !button->isEnabled() || QQuickAbstractButtonPrivate::get(button)->touchId != -1)
         return;
 
     QQuickMenuItem *oldCurrentItem = currentItem;
@@ -689,18 +706,23 @@ void QQuickMenuPrivate::contentData_append(QQmlListProperty<QObject> *prop, QObj
     }
 }
 
-int QQuickMenuPrivate::contentData_count(QQmlListProperty<QObject> *prop)
+qsizetype QQuickMenuPrivate::contentData_count(QQmlListProperty<QObject> *prop)
 {
     QQuickMenu *q = static_cast<QQuickMenu *>(prop->object);
     return QQuickMenuPrivate::get(q)->contentData.count();
 }
 
-QObject *QQuickMenuPrivate::contentData_at(QQmlListProperty<QObject> *prop, int index)
+QObject *QQuickMenuPrivate::contentData_at(QQmlListProperty<QObject> *prop, qsizetype index)
 {
     QQuickMenu *q = static_cast<QQuickMenu *>(prop->object);
     return QQuickMenuPrivate::get(q)->contentData.value(index);
 }
 
+QPalette QQuickMenuPrivate::defaultPalette() const
+{
+    return QQuickTheme::palette(QQuickTheme::Menu);
+}
+
 void QQuickMenuPrivate::contentData_clear(QQmlListProperty<QObject> *prop)
 {
     QQuickMenu *q = static_cast<QQuickMenu *>(prop->object);
@@ -712,9 +734,20 @@ QQuickMenu::QQuickMenu(QObject *parent)
 {
     Q_D(QQuickMenu);
     setFocus(true);
+    d->init();
     connect(d->contentModel, &QQmlObjectModel::countChanged, this, &QQuickMenu::countChanged);
 }
 
+QQuickMenu::~QQuickMenu()
+{
+    Q_D(QQuickMenu);
+    // We have to do this to ensure that the change listeners are removed.
+    // It's too late to do this in ~QQuickMenuPrivate, as contentModel has already
+    // been destroyed before that is called.
+    while (d->contentModel->count() > 0)
+        d->removeItem(0, d->itemAt(0));
+}
+
 /*!
     \qmlmethod Item QtQuick.Controls::Menu::itemAt(int index)
 
@@ -780,23 +813,6 @@ void QQuickMenu::moveItem(int from, int to)
         d->moveItem(from, to);
 }
 
-/*!
-    \deprecated
-    \qmlmethod void QtQuick.Controls::Menu::removeItem(int index)
-
-    Use Menu::removeItem(Item) or Menu::takeItem(int) instead.
-*/
-void QQuickMenu::removeItem(const QVariant &var)
-{
-    if (var.userType() == QMetaType::Nullptr)
-        return;
-
-    if (QQuickItem *item = var.value<QQuickItem *>())
-        removeItem(item);
-    else
-        takeItem(var.toInt());
-}
-
 /*!
     \since QtQuick.Controls 2.3 (Qt 5.10)
     \qmlmethod void QtQuick.Controls::Menu::removeItem(Item item)
@@ -1475,6 +1491,15 @@ void QQuickMenu::keyPressEvent(QKeyEvent *event)
             d->propagateKeyEvent(event);
         break;
 
+#if QT_CONFIG(shortcut)
+    case Qt::Key_Alt:
+        // If &mnemonic shortcut is enabled, go back to (possibly) the parent
+        // menu bar so the shortcut key will be processed by the menu bar.
+        if (!QKeySequence::mnemonic(QStringLiteral("&A")).isEmpty())
+            close();
+        break;
+#endif
+
     default:
         break;
     }
@@ -1497,11 +1522,6 @@ QFont QQuickMenu::defaultFont() const
     return QQuickTheme::font(QQuickTheme::Menu);
 }
 
-QPalette QQuickMenu::defaultPalette() const
-{
-    return QQuickTheme::palette(QQuickTheme::Menu);
-}
-
 #if QT_CONFIG(accessibility)
 QAccessible::Role QQuickMenu::accessibleRole() const
 {
